﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Linq;
using System.ServiceProcess;
using System.Text;
using System.Configuration;
using System.Threading;
using System.IO;

namespace Derrick.Web
{
    public partial class CometService : ServiceBase
    {
        public static AppConfig Configuration;

        private static WIServer _wiserver;
        private static SIServer _siserver;
        private static MessageLoop _messageQueueHandler;

        private TraceSwitch _traceSwitch;

        public CometService()
        {
            InitializeComponent();
        }

        protected override void OnStart(string[] args)
        {

            System.GC.Collect();

            string configfile = "";

            //Set up tracing:
            _traceSwitch = new TraceSwitch("TraceLevelSwitch", "Tracing level of COMET server.");

            Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Derrick COMET service OnStart() called...");

            string cometSleepTimeStr = ConfigurationManager.AppSettings["CometSleepTime"];
            int cometSleepTime = 5000;
            if (!String.IsNullOrEmpty(cometSleepTimeStr))
            {
                cometSleepTime = Int32.Parse(cometSleepTimeStr);
            }
            Thread.Sleep(cometSleepTime);

            try
            {

                if (args.Length == 0)
                {
                    configfile = ConfigurationManager.AppSettings["CometConfig"];
                    if (String.IsNullOrEmpty(configfile))
                    {
                        configfile = "cometbox.conf";
                    }
                }
                else
                {
                    configfile = args[0];
                }

                if (!new FileInfo(configfile).Exists)
                {
                    try
                    {
                        AppConfig.BuildNewConfigFile(configfile);
                    }
                    catch (Exception ex)
                    {
                        Trace.WriteLineIf(_traceSwitch.TraceError, ex.ToString(), "CometService");
                    }
                }

                if (new FileInfo(configfile).Exists)
                {
                    if ((Configuration = AppConfig.LoadConfig(configfile)) == null)
                    {
                        //Console.WriteLine("Error loading configuration.");
                        GracefullyChokeAndDie();
                        return;
                    }
                }
                else
                {
                    //Console.WriteLine("Configuration not found: " + configfile);
                    GracefullyChokeAndDie();
                    return;
                }


                _wiserver = new WIServer(Configuration.WebInterface);
                _siserver = new SIServer(Configuration.ServerInterface);
                _messageQueueHandler = new MessageLoop();
                _messageQueueHandler.SendMessageEvent += new MessageLoop.SendMessageEventHandler(_siserver.BroadcastMessage);
                _messageQueueHandler.SendMessageEvent += new MessageLoop.SendMessageEventHandler(_wiserver.BroadcastMessage);

                Trace.WriteLineIf(_traceSwitch.TraceVerbose, "Derrick COMET service OnStart() completed.");

            }
            catch (Exception ex)
            {
                Trace.WriteLineIf(_traceSwitch.TraceError, ex.ToString());
            }

        }

        protected override void OnStop()
        {
            //_wiserver.CloseListener();
            //_siserver.CloseListener();


            _wiserver.ShutDown();
            _siserver.ShutDown();
            GracefullyChokeAndDie();
        }

        static void GracefullyChokeAndDie()
        {
            //Console.WriteLine("");
            //Console.WriteLine("Press any key to exit...");
            //Console.ReadKey(true);
            System.GC.Collect();
            Trace.WriteLine("Gracefully exiting...");
            Trace.Flush();
            Trace.Close();
        }

    }
}
