using System;
using System.Threading;
using System.Text;
using System.Net;
using System.Net.Sockets;
using System.IO;
using System.Xml;
using System.Xml.Serialization;
using System.Diagnostics;

namespace Derrick.Web
{
    public class AppConfig
    {
        public ServerInterfaceConfig ServerInterface;
        public ClientInterfaceConfig ClientInterface;
        public WebInterfaceConfig WebInterface;

        public static AppConfig LoadConfig(string file)
        {
            TraceSwitch _traceSwitch = new TraceSwitch("TraceLevelSwitch", "Comet Trace");

            AppConfig c = null;
            try
            {
                using (FileStream fs = new FileStream(file, FileMode.Open))
                {
                    XmlSerializer serializer = new XmlSerializer(typeof(AppConfig));
                    c = (AppConfig)serializer.Deserialize(fs);
                }
            }
            catch (Exception ex)
            {
                Trace.WriteLineIf(_traceSwitch.TraceError, ex.ToString());
            }

            return c;
        }

        public static void BuildNewConfigFile(string file)
        {
            AppConfig c = new AppConfig();

            c.ClientInterface = new ClientInterfaceConfig();
            c.ClientInterface.Authentication = new AuthConfig();
            c.ClientInterface.Authentication.Type = AuthType.None;
            c.ClientInterface.Authentication.Realm = "Derrick Comet Client Interface";
            c.ClientInterface.Authentication.Username = "none";
            c.ClientInterface.Authentication.Password = "none";

            c.WebInterface = new WebInterfaceConfig();
            c.WebInterface.Authentication = new AuthConfig();
            c.WebInterface.Authentication.Type = AuthType.Basic;
            c.WebInterface.Authentication.Realm = "Derrick Comet Web Interface";
            c.WebInterface.Authentication.Username = "cometbox";
            c.WebInterface.Authentication.Password = "cometbox123";

            c.ServerInterface = new ServerInterfaceConfig();
            c.ServerInterface.Authentication = new AuthConfig();
            c.ServerInterface.Authentication.Type = AuthType.Basic;
            c.ServerInterface.Authentication.Realm = "Derrick Comet Server Interface";
            c.ServerInterface.Authentication.Username = "server";
            c.ServerInterface.Authentication.Password = "server123";
            c.ServerInterface.AcceptedIPs = new string[] {"127.0.0.1"};

            using (FileStream fs = new FileStream(file, FileMode.Create)) {
                XmlSerializer serializer = new XmlSerializer(typeof(AppConfig));
                serializer.Serialize(fs, c);
                fs.Close();
            }
        }
    }
}
